@@ -1,14 +1,17 @@
 import json
 from pathlib import Path
-import tomli
-from typing import Any, Dict, Mapping, Optional, Sequence, Tuple, Union
+
+try:
+    import tomllib as tomli
+except ImportError:
+    import tomli  # type: ignore[no-redef]
+
+from typing import Any, Dict, List, Mapping, Optional, Sequence, Tuple, Union
+
 from .exceptions import PoeException
 
 
 class PoeConfig:
-    _table: Mapping[str, Any]
-
-    TOML_NAME = "pyproject.toml"
     KNOWN_SHELL_INTERPRETERS = (
         "posix",
         "sh",
@@ -28,9 +31,9 @@ class PoeConfig:
         "default_array_task_type": str,
         "default_array_item_task_type": str,
         "env": dict,
-        "envfile": str,
+        "envfile": (str, list),
         "executor": dict,
-        "include": (str, list),
+        "include": (str, list, dict),
         "poetry_command": str,
         "poetry_hooks": dict,
         "shell_interpreter": (str, list),
@@ -46,9 +49,11 @@ class PoeConfig:
         self,
         cwd: Optional[Union[Path, str]] = None,
         table: Optional[Mapping[str, Any]] = None,
+        config_name: str = "pyproject.toml",
     ):
-        self.cwd = Path(".").resolve() if cwd is None else Path(cwd)
+        self.cwd = Path().resolve() if cwd is None else Path(cwd)
         self._poe = {} if table is None else dict(table)
+        self._config_name = config_name
         self._project_dir: Optional[Path] = None
 
     @property
@@ -100,21 +105,22 @@ class PoeConfig:
 
     def load(self, target_dir: Optional[str] = None):
         if self._poe:
-            raise PoeException("Cannot load poetry config more than once!")
-        config_path = self.find_pyproject_toml(target_dir)
+            return
+
+        config_path = self.find_config_file(target_dir)
         try:
-            self._project = self._read_pyproject(config_path)
+            self._project = self._read_config_file(config_path)
             self._poe = self._project["tool"]["poe"]
-        except KeyError as error:
+        except KeyError:
             raise PoeException(
-                f"No poe configuration found in file at {self.TOML_NAME}"
+                f"No poe configuration found in file at {self._config_name}"
             )
         self._project_dir = config_path.parent
         self._load_includes(self._project_dir)
 
     def validate(self):
-        from .task import PoeTask
         from .executor import PoeExecutor
+        from .task import PoeTask
 
         # Validate keys
         supported_keys = {"tasks", *self.__options__}
@@ -181,7 +187,7 @@ class PoeConfig:
         # Validate shell_interpreter type
         for interpreter in self.shell_interpreter:
             if interpreter not in self.KNOWN_SHELL_INTERPRETERS:
-                return (
+                raise PoeException(
                     f"Unsupported value {interpreter!r} for option `shell_interpreter`."
                 )
 
@@ -192,106 +198,133 @@ class PoeConfig:
                 "Should be between -1 and 2."
             )
 
-    def find_pyproject_toml(self, target_dir: Optional[str] = None) -> Path:
+    def find_config_file(self, target_dir: Optional[str] = None) -> Path:
         """
-        Resolve a path to a pyproject.toml using one of two strategies:
-          1. If target_dir is provided then only look there, (accept path to .toml file
-             or to a directory dir).
-          2. Otherwise look for the pyproject.toml is the current working directory,
+        Resolve a path to a self._config_name using one of two strategies:
+          1. If target_dir is provided then only look there, (accept path to config file
+             or to a directory).
+          2. Otherwise look for the self._config_name in the current working directory,
              following by all parent directories in ascending order.
 
         Both strategies result in an Exception on failure.
         """
         if target_dir:
             target_path = Path(target_dir).resolve()
-            if not target_path.name.endswith(".toml"):
-                target_path = target_path.joinpath(self.TOML_NAME)
+            if not (
+                target_path.name.endswith(".toml") or target_path.name.endswith(".json")
+            ):
+                target_path = target_path.joinpath(self._config_name)
             if not target_path.exists():
                 raise PoeException(
-                    "Poe could not find a pyproject.toml file at the given location: "
-                    f"{target_dir}"
+                    f"Poe could not find a {self._config_name} file at the given "
+                    f"location: {target_dir}"
                 )
             return target_path
 
-        maybe_result = self.cwd.joinpath(self.TOML_NAME)
+        maybe_result = self.cwd.joinpath(self._config_name)
         while not maybe_result.exists():
-            if maybe_result.parent == Path("/"):
+            if len(maybe_result.parents) == 1:
                 raise PoeException(
-                    f"Poe could not find a pyproject.toml file in {self.cwd} or"
+                    f"Poe could not find a {self._config_name} file in {self.cwd} or"
                     " its parents"
                 )
-            maybe_result = maybe_result.parents[1].joinpath(self.TOML_NAME).resolve()
+            maybe_result = maybe_result.parents[1].joinpath(self._config_name).resolve()
         return maybe_result
 
     def _load_includes(self, project_dir: Path):
-        includes: Union[str, Sequence[str]] = self._poe.get("include", tuple())
-        if isinstance(includes, str):
-            includes = (includes,)
+        include_option: Union[str, Sequence[str]] = self._poe.get("include", tuple())
+        includes: List[Dict[str, str]] = []
 
-        for path_str in includes:
-            if not isinstance(path_str, str):
-                raise PoeException(f"Invalid include value {path_str!r}")
+        if isinstance(include_option, str):
+            includes.append({"path": include_option})
+        elif isinstance(include_option, dict):
+            includes.append(include_option)
+        elif isinstance(include_option, list):
+            valid_keys = {"path", "cwd"}
+            for include in include_option:
+                if isinstance(include, str):
+                    includes.append({"path": include})
+                elif (
+                    isinstance(include, dict)
+                    and include.get("path")
+                    and set(include.keys()) <= valid_keys
+                ):
+                    includes.append(include)
+                else:
+                    raise PoeException(
+                        f"Invalid item for the include option {include!r}"
+                    )
 
-            include_path = project_dir.joinpath(path_str).resolve()
-
-            if not str(include_path).startswith(str(project_dir)):
-                raise PoeException(
-                    f"Cannot include config from {path_str!r} because it's outside of "
-                    "the project."
-                )
+        for include in includes:
+            include_path = project_dir.joinpath(include["path"]).resolve()
 
             if not include_path.exists():
                 # TODO: print warning in verbose mode, requires access to ui somehow
                 continue
 
-            if include_path.name.endswith(".toml"):
-                try:
-                    with include_path.open("rb") as file:
-                        self._merge_config(tomli.load(file), include_path)
-                except tomli.TOMLDecodeError as error:
-                    raise PoeException(
-                        f"Couldn't parse included toml file from {include_path}", error
-                    ) from error
+            try:
+                include_config = PoeConfig(
+                    cwd=include.get("cwd", self.project_dir),
+                    table=self._read_config_file(include_path)["tool"]["poe"],
+                )
+                include_config._project_dir = self._project_dir
+            except (PoeException, KeyError) as error:
+                raise PoeException(
+                    f"Invalid content in included file from {include_path}", error
+                ) from error
 
-            elif include_path.name.endswith(".json"):
-                try:
-                    with include_path.open("rb") as file:
-                        self._merge_config(json.load(file), include_path)
-                except json.decoder.JSONDecodeError as error:
-                    raise PoeException(
-                        f"Couldn't parse included json file from {include_path}", error
-                    ) from error
+            self._merge_config(include_config)
 
-    def _merge_config(self, extra_config: Mapping[str, Any], path: Path):
+    def _merge_config(self, include_config: "PoeConfig"):
         from .task import PoeTask
 
-        try:
-            poe_config = extra_config.get("tool", {}).get("poe", {})
-            tasks = poe_config.get("tasks", {})
-        except AttributeError as error:
-            raise PoeException(
-                f"Invalid content in included file from {path}", error
-            ) from error
-
         # Env is special because it can be extended rather than just overwritten
-        if "env" in poe_config:
-            self._poe["env"] = {**poe_config["env"], **self._poe.get("env", {})}
+        if include_config.global_env:
+            self._poe["env"] = {**include_config.global_env, **self._poe.get("env", {})}
 
-        if "envfile" in poe_config and "envfile" not in self._poe:
-            self._poe["envfile"] = poe_config["envfile"]
+        if include_config.global_envfile and "envfile" not in self._poe:
+            self._poe["envfile"] = include_config.global_envfile
 
         # Includes additional tasks with preserved ordering
         self._poe["tasks"] = own_tasks = self._poe.get("tasks", {})
-        for task_name, task_def in tasks.items():
-            if task_name not in own_tasks:
-                own_tasks[task_name] = task_def
+        for task_name, task_def in include_config.tasks.items():
+            if task_name in own_tasks:
+                # don't override tasks from the base config
+                continue
+
+            task_def = PoeTask.normalize_task_def(task_def, include_config)
+            if include_config.cwd:
+                # Override the config of each task to use the include level cwd as a
+                # base for the task level cwd
+                if "cwd" in task_def:
+                    # rebase the configured cwd onto the include level cwd
+                    task_def["cwd"] = str(
+                        Path(include_config.cwd)
+                        .resolve()
+                        .joinpath(task_def["cwd"])
+                        .relative_to(self.project_dir)
+                    )
+                else:
+                    task_def["cwd"] = str(include_config.cwd)
+
+            own_tasks[task_name] = task_def
 
     @staticmethod
-    def _read_pyproject(path: Path) -> Mapping[str, Any]:
+    def _read_config_file(path: Path) -> Mapping[str, Any]:
         try:
             with path.open("rb") as file:
-                return tomli.load(file)
+                if path.suffix.endswith(".json"):
+                    return json.load(file)
+                else:
+                    return tomli.load(file)
+
         except tomli.TOMLDecodeError as error:
             raise PoeException(f"Couldn't parse toml file at {path}", error) from error
+
+        except json.decoder.JSONDecodeError as error:
+            raise PoeException(
+                f"Couldn't parse json file from {path}", error
+            ) from error
+
         except Exception as error:
             raise PoeException(f"Couldn't open file at {path}") from error
