@@ -1,7 +1,9 @@
-import signal
-from subprocess import Popen, PIPE
+import os
+import shutil
 import sys
+from pathlib import Path
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     Mapping,
@@ -10,15 +12,15 @@ from typing import (
     Sequence,
     Tuple,
     Type,
-    TYPE_CHECKING,
     Union,
 )
-from ..exceptions import PoeException
-from ..virtualenv import Virtualenv
+
+from ..exceptions import ExecutionError, PoeException
 
 if TYPE_CHECKING:
-    from pathlib import Path
     from ..context import RunContext
+    from ..env.manager import EnvVarsManager
+
 
 # TODO: maybe invert the control so the executor is given a task to run?
 
@@ -43,17 +45,18 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
     A base class for poe task executors
     """
 
-    working_dir: Optional["Path"]
+    working_dir: Optional[Path]
 
     __executor_types: Dict[str, Type["PoeExecutor"]] = {}
+    __key__: Optional[str] = None
 
     def __init__(
         self,
         invocation: Tuple[str, ...],
         context: "RunContext",
         options: Mapping[str, str],
-        env: Mapping[str, str],
-        working_dir: Optional["Path"] = None,
+        env: "EnvVarsManager",
+        working_dir: Optional[Path] = None,
         dry: bool = False,
         capture_stdout: Union[str, bool] = False,
     ):
@@ -63,15 +66,26 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
         self.working_dir = working_dir
         self.env = env
         self.dry = dry
-        self.capture_stdout = capture_stdout
+        self.capture_stdout = (
+            Path(self.context.config.project_dir).joinpath(
+                self.env.fill_template(capture_stdout)
+            )
+            if isinstance(capture_stdout, str)
+            else capture_stdout
+        )
+        self._is_windows = sys.platform == "win32"
+
+    @classmethod
+    def works_with_context(cls, context: "RunContext") -> bool:
+        return True
 
     @classmethod
     def get(
         cls,
         invocation: Tuple[str, ...],
         context: "RunContext",
-        env: Mapping[str, str],
-        working_dir: Optional["Path"] = None,
+        env: "EnvVarsManager",
+        working_dir: Optional[Path] = None,
         dry: bool = False,
         executor_config: Optional[Mapping[str, str]] = None,
         capture_stdout: Union[str, bool] = False,
@@ -92,6 +106,7 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
         by making some reasonable assumptions based on visible features of the
         environment
         """
+
         config_executor_type = context.executor_type
         if executor_config:
             if executor_config["type"] not in cls.__executor_types:
@@ -100,24 +115,103 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
                 )
             return cls.__executor_types[executor_config["type"]]
         elif config_executor_type == "auto":
-            if "poetry" in context.config.project["tool"]:
-                # Looks like this is a poetry project!
-                return cls.__executor_types["poetry"]
+            for impl in [
+                cls.__executor_types["poetry"],
+                cls.__executor_types["virtualenv"],
+            ]:
+                if impl.works_with_context(context):
+                    return impl
 
-            if Virtualenv.detect(context.project_dir):
-                # Looks like there's a local virtualenv
-                return cls.__executor_types["virtualenv"]
             # Fallback to not using any particular environment
             return cls.__executor_types["simple"]
         else:
             if config_executor_type not in cls.__executor_types:
                 raise PoeException(
-                    f"Cannot instantiate unknown executor" + repr(config_executor_type)
+                    "Cannot instantiate unknown executor" + repr(config_executor_type)
                 )
             return cls.__executor_types[config_executor_type]
 
-    def execute(self, cmd: Sequence[str], input: Optional[bytes] = None) -> int:
-        raise NotImplementedError
+    def execute(
+        self, cmd: Sequence[str], input: Optional[bytes] = None, use_exec: bool = False
+    ) -> int:
+        """
+        Execute the given cmd.
+        """
+
+        # Attempt to explicitly resolve the target executable, because we can't count
+        # on the OS to do this consistently.
+        resolved_executable = shutil.which(cmd[0])
+        if resolved_executable:
+            cmd = (resolved_executable, *cmd[1:])
+
+        return self._execute_cmd(cmd, input=input, use_exec=use_exec)
+
+    def _execute_cmd(
+        self,
+        cmd: Sequence[str],
+        *,
+        input: Optional[bytes] = None,
+        env: Optional[Mapping[str, str]] = None,
+        shell: bool = False,
+        use_exec: bool = False,
+    ) -> int:
+        """
+        Execute the given cmd either as a subprocess or use exec to replace the current
+        process. Using exec supports fewer options, and doesn't work on windows.
+        """
+
+        try:
+            if self.working_dir and not self.working_dir.is_dir():
+                raise PoeException(
+                    f"Working directory {self.working_dir} could not be found."
+                )
+            if use_exec:
+                if input:
+                    raise ExecutionError("Cannot exec task that requires input!")
+                if shell:
+                    raise ExecutionError("Cannot exec task that requires shell!")
+                if not self._is_windows:
+                    # execvpe doesn't work properly on windows so we just don't go there
+                    return self._exec(cmd, env=env)
+
+            return self._exec_via_subproc(cmd, input=input, env=env, shell=shell)
+        except FileNotFoundError as error:
+            if error.filename == cmd[0]:
+                return self._handle_file_not_found(cmd, error)
+            if error.filename == self.working_dir:
+                raise PoeException(
+                    "The specified working directory does not exists "
+                    f"'{self.working_dir}'"
+                )
+            raise
+
+    def _handle_file_not_found(
+        self, cmd: Sequence[str], error: FileNotFoundError
+    ) -> int:
+        raise PoeException(f"executable {cmd[0]!r} could not be found") from error
+
+    def _exec(
+        self,
+        cmd: Sequence[str],
+        *,
+        env: Optional[Mapping[str, str]] = None,
+    ):
+        if self.dry:
+            return 0
+
+        # Beware: this is the point of no return!
+
+        exec_env = dict(
+            (self.env.to_dict() if env is None else env), POE_ACTIVE=self.__key__
+        )
+        if self.working_dir:
+            os.chdir(self.working_dir)
+        sys.stdout.flush()
+
+        # if running tests then wrap up coverage instrumentation while we still can
+        _stop_coverage()
+
+        os.execvpe(cmd[0], tuple(cmd), exec_env)
 
     def _exec_via_subproc(
         self,
@@ -127,17 +221,27 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
         env: Optional[Mapping[str, str]] = None,
         shell: bool = False,
     ) -> int:
+        import signal
+        from subprocess import PIPE, Popen
+
         if self.dry:
             return 0
         popen_kwargs: MutableMapping[str, Any] = {"shell": shell}
-        popen_kwargs["env"] = self.env if env is None else env
+        popen_kwargs["env"] = dict(
+            (self.env.to_dict() if env is None else env), POE_ACTIVE=self.__key__
+        )
         if input is not None:
             popen_kwargs["stdin"] = PIPE
         if self.capture_stdout:
-            if isinstance(self.capture_stdout, str):
+            if isinstance(self.capture_stdout, Path):
+                # ruff: noqa: SIM115
                 popen_kwargs["stdout"] = open(self.capture_stdout, "wb")
             else:
                 popen_kwargs["stdout"] = PIPE
+
+            if "PYTHONIOENCODING" not in popen_kwargs["env"]:
+                popen_kwargs["env"]["PYTHONIOENCODING"] = "utf-8"
+
         if self.working_dir is not None:
             popen_kwargs["cwd"] = self.working_dir
 
@@ -147,19 +251,21 @@ class PoeExecutor(metaclass=MetaPoeExecutor):
         proc = Popen(cmd, **popen_kwargs)
 
         # signal pass through
-        def handle_signal(signum, _frame):
+        def handle_sigint(signum, _frame):
+            # sigint is not handled on windows
+            signum = signal.CTRL_C_EVENT if self._is_windows else signum
             proc.send_signal(signum)
 
-        old_signal_handler = signal.signal(signal.SIGINT, handle_signal)
+        old_sigint_handler = signal.signal(signal.SIGINT, handle_sigint)
 
         # send data to the subprocess and wait for it to finish
         (captured_stdout, _) = proc.communicate(input)
 
-        if self.capture_stdout == True:
-            self.context.captured_stdout[self.invocation] = captured_stdout.decode()
+        if self.capture_stdout is True:
+            self.context.save_task_output(self.invocation, captured_stdout)
 
         # restore signal handler
-        signal.signal(signal.SIGINT, old_signal_handler)
+        signal.signal(signal.SIGINT, old_sigint_handler)
 
         return proc.returncode
 
