@@ -1,7 +1,11 @@
-from typing import Any, Dict, Optional, Sequence, Type
-from .base import PoeExecutor
+from typing import TYPE_CHECKING, Any, Dict, Optional, Sequence, Type
+
 from ..exceptions import PoeException
-from ..virtualenv import Virtualenv
+from .base import PoeExecutor
+
+if TYPE_CHECKING:
+    from ..context import RunContext
+    from ..virtualenv import Virtualenv
 
 
 class VirtualenvExecutor(PoeExecutor):
@@ -12,20 +16,39 @@ class VirtualenvExecutor(PoeExecutor):
     __key__ = "virtualenv"
     __options__: Dict[str, Type] = {"location": str}
 
-    def execute(self, cmd: Sequence[str], input: Optional[bytes] = None) -> int:
+    @classmethod
+    def works_with_context(cls, context: "RunContext") -> bool:
+        from ..virtualenv import Virtualenv
+
+        return Virtualenv.detect(context.project_dir)
+
+    def execute(
+        self, cmd: Sequence[str], input: Optional[bytes] = None, use_exec: bool = False
+    ) -> int:
         """
         Execute the given cmd as a subprocess inside the configured virtualenv
         """
         venv = self._resolve_virtualenv()
-        return self._exec_via_subproc(
+
+        return self._execute_cmd(
             (venv.resolve_executable(cmd[0]), *cmd[1:]),
             input=input,
-            env=dict(
-                venv.get_env_vars(self.env), POE_ACTIVE=VirtualenvExecutor.__key__
-            ),
+            env=venv.get_env_vars(self.env.to_dict()),
+            use_exec=use_exec,
         )
 
-    def _resolve_virtualenv(self) -> Virtualenv:
+    def _handle_file_not_found(
+        self, cmd: Sequence[str], error: FileNotFoundError
+    ) -> int:
+        venv = self._resolve_virtualenv()
+        error_context = f" using virtualenv {str(venv.path)!r}" if venv else ""
+        raise PoeException(
+            f"executable {cmd[0]!r} could not be found{error_context}"
+        ) from error
+
+    def _resolve_virtualenv(self) -> "Virtualenv":
+        from ..virtualenv import Virtualenv
+
         if "location" in self.options:
             venv = Virtualenv(
                 self.context.project_dir.joinpath(self.options["location"])
@@ -55,7 +78,10 @@ class VirtualenvExecutor(PoeExecutor):
         Validate that location is a string if given and no other options are given.
         """
         if "location" in config and not isinstance(config["location"], str):
-            return f"The location option virtualenv executor must be a string not: {config['location']!r}"
+            return (
+                "The location option virtualenv executor must be a string not: "
+                f"{config['location']!r}"
+            )
         extra_options = set(config.keys()) - {"type", "location"}
         if extra_options:
             return f"Unexpected keys for executor config: {extra_options!r}"
