@@ -1,20 +1,22 @@
 # pylint: disable=import-error
 
+from pathlib import Path
+from typing import Any, Dict, List
+
 from cleo.commands.command import Command
 from cleo.events.console_command_event import ConsoleCommandEvent
-from cleo.events.event_dispatcher import EventDispatcher
 from cleo.events.console_events import COMMAND, TERMINATE
+from cleo.events.event_dispatcher import EventDispatcher
 from cleo.io.io import IO
-from pathlib import Path
-from poetry.console.application import Application, COMMANDS
+from poetry.console.application import COMMANDS, Application
 from poetry.plugins.application_plugin import ApplicationPlugin
-from typing import Any, Dict, List
 
 from .exceptions import PoePluginException
 
 
 class PoeCommand(Command):
     prefix: str
+    command_prefix: str = "poe"
 
     def __init__(self):
         super().__init__()
@@ -51,10 +53,10 @@ class PoeCommand(Command):
 
             poetry_env_path = EnvManager(application.poetry).get().path
         # pylint: disable=bare-except
-        except:
+        except:  # noqa: E722
             poetry_env_path = None
 
-        cwd = Path(".").resolve()
+        cwd = Path().resolve()
         config = PoeConfig(cwd=cwd)
 
         if io.output.is_quiet():
@@ -69,13 +71,42 @@ class PoeCommand(Command):
             config=config,
             output=io.output.stream,
             poetry_env_path=poetry_env_path,
+            program_name=f"poetry {cls.command_prefix}",
         )
 
 
 class PoetryPlugin(ApplicationPlugin):
     def activate(self, application: Application) -> None:
-        poe_config = self._get_config(application)
-        command_prefix = poe_config.get("poetry_command", "poe")
+        try:
+            return self._activate(application)
+
+        # pylint: disable=bare-except
+        except:  # noqa: E722
+            import os
+            import sys
+
+            debug = bool(int(os.environ.get("DEBUG_POE_PLUGIN", "0")))
+            print(
+                "error: poethepoet plugin failed to activate."
+                + ("" if debug else " Set DEBUG_POE_PLUGIN=1 for details."),
+                file=sys.stderr,
+            )
+            if debug:
+                import traceback
+
+                traceback.print_exc()
+                raise SystemExit(1)
+
+    def _activate(self, application: Application) -> None:
+        try:
+            poe_config = self._get_config(application)
+        except RuntimeError:
+            # If there's no pyproject.toml then probably that's OK, don't freak out
+            return
+
+        command_prefix = poe_config.get("poetry_command", "poe").strip()
+        PoeCommand.command_prefix = command_prefix
+
         poe_tasks = poe_config.get("tasks", {})
         self._validate_command_prefix(command_prefix)
 
@@ -97,10 +128,12 @@ class PoetryPlugin(ApplicationPlugin):
             self._register_command(
                 application,
                 "",
-                {"help": "A task runner that works well with poetry"},
+                {"help": "Run poe tasks defined for this project"},
                 command_prefix,
             )
             for task_name, task in poe_tasks.items():
+                if task_name.startswith("_"):
+                    continue
                 self._register_command(
                     application, task_name, task, f"{command_prefix} "
                 )
@@ -117,15 +150,14 @@ class PoetryPlugin(ApplicationPlugin):
             pyproject = application.poetry.pyproject.data
 
         # pylint: disable=bare-except
-        except:
+        except:  # noqa: E722
             # Fallback to loading the config again in case of future failure of the
             # above undocumented API
             import tomlkit
+
             from .config import PoeConfig
 
-            pyproject = tomlkit.loads(
-                Path(PoeConfig().find_pyproject_toml()).read_text()
-            )
+            pyproject = tomlkit.loads(Path(PoeConfig().find_config_file()).read_text())
 
         return pyproject.get("tool", {}).get("poe", {})
 
@@ -192,7 +224,7 @@ class PoetryPlugin(ApplicationPlugin):
             import shlex
 
             task_status = PoeCommand.get_poe(application, event.io)(
-                cli_args=shlex.split(task)
+                cli_args=shlex.split(task), internal=True
             )
 
             if task_status:
@@ -207,7 +239,7 @@ class PoetryPlugin(ApplicationPlugin):
         """
         Cleo is quite opinionated about CLI structure and loose about how options are
         used, and so doesn't currently support invidual commands having their own way of
-        interpreting arguments, and forces them in inherit certain options from the
+        interpreting arguments, and forces them to inherit certain options from the
         application. This is a problem for poe which requires that global options are
         provided before the task name, and everything after the task name is interpreted
         ONLY in terms of the task.
@@ -217,7 +249,7 @@ class PoetryPlugin(ApplicationPlugin):
         option parsing are effectively disabled following any occurance of a "--" on the
         command line, but parsing of the command name still works! Thus the solution is
         to detect when it is a command from this plugin that is about to be executed and
-        insert the "--" token and the start of the tokens list of the ArgvInput instance
+        insert the "--" token at the start of the tokens list of the ArgvInput instance
         that the application is about to read the CLI options from.
 
         Hopefully this doesn't get broken by a future update to poetry or cleo :S
@@ -241,7 +273,7 @@ class PoetryPlugin(ApplicationPlugin):
                 # update tokens list in place
                 tokens.insert(task_name_index, "--")
 
-            continue_run(self, io)
+            return continue_run(self, io)
 
         # Apply the patch
         cleo.application.Application._run = _run
