@@ -1,5 +1,5 @@
-import argparse
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     List,
@@ -9,11 +9,12 @@ from typing import (
     Tuple,
     Type,
     Union,
-    TYPE_CHECKING,
 )
 
 if TYPE_CHECKING:
-    from ..config import PoeConfig
+    from argparse import ArgumentParser
+
+    from ..env.manager import EnvVarsManager
 
 ArgParams = Dict[str, Any]
 ArgsDef = Union[List[str], List[ArgParams], Dict[str, ArgParams]]
@@ -26,6 +27,7 @@ arg_param_schema: Dict[str, Union[Type, Tuple[Type, ...]]] = {
     "positional": (bool, str),
     "required": bool,
     "type": str,
+    "multiple": (bool, int),
 }
 arg_types: Dict[str, Type] = {
     "string": str,
@@ -38,9 +40,17 @@ arg_types: Dict[str, Type] = {
 class PoeTaskArgs:
     _args: Tuple[ArgParams, ...]
 
-    def __init__(self, args_def: ArgsDef, task_name: str):
+    def __init__(
+        self,
+        args_def: ArgsDef,
+        task_name: str,
+        program_name: str,
+        env: "EnvVarsManager",
+    ):
         self._args = self._normalize_args_def(args_def)
+        self._program_name = program_name
         self._task_name = task_name
+        self._env = env
 
     @classmethod
     def _normalize_args_def(cls, args_def: ArgsDef) -> Tuple[ArgParams, ...]:
@@ -85,17 +95,26 @@ class PoeTaskArgs:
     @classmethod
     def get_help_content(
         cls, args_def: Optional[ArgsDef]
-    ) -> List[Tuple[Tuple[str, ...], str]]:
+    ) -> List[Tuple[Tuple[str, ...], str, str]]:
         if args_def is None:
             return []
+
+        def format_default(arg) -> str:
+            default = arg.get("default")
+            if default:
+                return f"[default: {default}]"
+            return ""
+
         return [
-            (arg["options"], arg.get("help", ""))
+            (arg["options"], arg.get("help", ""), format_default(arg))
             for arg in cls._normalize_args_def(args_def)
         ]
 
     @classmethod
     def validate_def(cls, task_name: str, args_def: ArgsDef) -> Optional[str]:
         arg_names: Set[str] = set()
+        arg_params = []
+
         if isinstance(args_def, list):
             for item in args_def:
                 # can be a list of strings (just arg name) or ArgConfig dictionaries
@@ -103,14 +122,13 @@ class PoeTaskArgs:
                     arg_name = item
                 elif isinstance(item, dict):
                     arg_name = item.get("name", "")
-                    error = cls._validate_params(item, arg_name, task_name)
-                    if error:
-                        return error
+                    arg_params.append((item, arg_name, task_name))
                 else:
                     return f"Arg {item!r} of task {task_name!r} has invlaid type"
                 error = cls._validate_name(arg_name, task_name, arg_names)
                 if error:
                     return error
+
         elif isinstance(args_def, dict):
             for arg_name, params in args_def.items():
                 error = cls._validate_name(arg_name, task_name, arg_names)
@@ -121,12 +139,27 @@ class PoeTaskArgs:
                         f"Unexpected 'name' option for arg {arg_name!r} of task "
                         f"{task_name!r}"
                     )
-                error = cls._validate_params(params, arg_name, task_name)
-                if error:
-                    return error
-                error = cls._validate_type(params, arg_name, task_name)
-                if error:
-                    return error
+                arg_params.append((params, arg_name, task_name))
+
+        positional_multiple = None
+        for params, arg_name, task_name in arg_params:
+            error = cls._validate_type(params, arg_name, task_name)
+            if error:
+                return error
+
+            error = cls._validate_params(params, arg_name, task_name)
+            if error:
+                return error
+
+            if params.get("positional", False):
+                if positional_multiple:
+                    return (
+                        f"Only the last positional arg of task {task_name!r} may accept"
+                        f" multiple values ({positional_multiple!r})."
+                    )
+                if params.get("multiple", False):
+                    positional_multiple = arg_name
+
         return None
 
     @classmethod
@@ -182,6 +215,22 @@ class PoeTaskArgs:
                     "https://docs.python.org/3/reference/lexical_analysis.html#identifiers"
                 )
 
+        multiple = params.get("multiple", False)
+        if (
+            not isinstance(multiple, bool)
+            and isinstance(multiple, int)
+            and multiple < 2
+        ):
+            return (
+                f"The multiple option for arg {arg_name!r} of {task_name!r}"
+                " must be given a boolean or integer >= 2"
+            )
+        if multiple is not False and params.get("type") == "boolean":
+            return (
+                "Incompatible param 'multiple' for arg {arg_name!r} of {task_name!r} "
+                "with type: 'boolean'"
+            )
+
         return None
 
     @classmethod
@@ -198,9 +247,13 @@ class PoeTaskArgs:
             )
         return None
 
-    def build_parser(self) -> argparse.ArgumentParser:
+    def build_parser(self) -> "ArgumentParser":
+        import argparse
+
         parser = argparse.ArgumentParser(
-            prog=f"poe {self._task_name}", add_help=False, allow_abbrev=False
+            prog=f"{self._program_name} {self._task_name}",
+            add_help=False,
+            allow_abbrev=False,
         )
         for arg in self._args:
             parser.add_argument(
@@ -211,16 +264,28 @@ class PoeTaskArgs:
 
     def _get_argument_params(self, arg: ArgParams):
         default = arg.get("default")
+        if isinstance(default, str):
+            default = self._env.fill_template(default)
+
         result = {
             "default": default,
             "help": arg.get("help", ""),
         }
 
         required = arg.get("required", False)
+        multiple = arg.get("multiple", False)
         arg_type = str(arg.get("type"))
 
+        if multiple is True:
+            if required:
+                result["nargs"] = "+"
+            else:
+                result["nargs"] = "*"
+        elif multiple and isinstance(multiple, int):
+            result["nargs"] = multiple
+
         if arg.get("positional", False):
-            if not required:
+            if not multiple and not required:
                 result["nargs"] = "?"
         else:
             result["dest"] = arg["name"]
@@ -233,13 +298,15 @@ class PoeTaskArgs:
 
         return result
 
-    def parse(self, extra_args: Sequence[str]):
-        parsed_args = vars(self.build_parser().parse_args(extra_args))
-        # Ensure positional args are still exposed by name even if the were parsed with
+    def parse(self, args: Sequence[str]) -> Dict[str, str]:
+        parsed_args = vars(self.build_parser().parse_args(args))
+
+        # Ensure positional args are still exposed by name even if they were parsed with
         # alternate identifiers
         for arg in self._args:
             if isinstance(arg.get("positional"), str):
                 parsed_args[arg["name"]] = parsed_args[arg["positional"]]
                 del parsed_args[arg["positional"]]
+
         # args named with dash case are converted to snake case before being exposed
         return {name.replace("-", "_"): value for name, value in parsed_args.items()}
