@@ -1,27 +1,27 @@
 import re
-import shlex
 import sys
+from os import environ
+from pathlib import Path
 from typing import (
+    TYPE_CHECKING,
     Any,
+    Collection,
     Dict,
     Iterator,
     List,
-    Mapping,
+    NamedTuple,
     Optional,
-    Sequence,
     Tuple,
     Type,
-    TYPE_CHECKING,
     Union,
 )
-from .args import PoeTaskArgs
+
 from ..exceptions import PoeException
-from ..helpers import is_valid_env_var
-from ..helpers.env import resolve_envvars
 
 if TYPE_CHECKING:
-    from ..context import RunContext
     from ..config import PoeConfig
+    from ..context import RunContext
+    from ..env.manager import EnvVarsManager
     from ..ui import PoeUi
 
 
@@ -48,19 +48,34 @@ class MetaPoeTask(type):
 TaskContent = Union[str, List[Union[str, Dict[str, Any]]]]
 
 
+class TaskInheritance(NamedTuple):
+    """
+    Collection of inheritanced config from a parent task to a child task
+    """
+
+    cwd: str
+
+    @classmethod
+    def from_task(cls, parent_task: "PoeTask"):
+        return cls(cwd=str(parent_task.options.get("cwd", parent_task.inheritance.cwd)))
+
+
 class PoeTask(metaclass=MetaPoeTask):
     name: str
     content: TaskContent
     options: Dict[str, Any]
+    inheritance: TaskInheritance
+    _parsed_args: Optional[Tuple[Dict[str, str], Tuple[str, ...]]] = None
 
     __options__: Dict[str, Union[Type, Tuple[Type, ...]]] = {}
     __content_type__: Type = str
     __base_options: Dict[str, Union[Type, Tuple[Type, ...]]] = {
         "args": (dict, list),
-        "capture_stdout": (str),
+        "capture_stdout": str,
+        "cwd": str,
         "deps": list,
         "env": dict,
-        "envfile": str,
+        "envfile": (str, list),
         "executor": dict,
         "help": str,
         "uses": dict,
@@ -80,18 +95,16 @@ class PoeTask(metaclass=MetaPoeTask):
         config: "PoeConfig",
         invocation: Tuple[str, ...],
         capture_stdout: bool = False,
+        inheritance: Optional[TaskInheritance] = None,
     ):
         self.name = name
-        self.content = content.strip() if isinstance(content, str) else content
-        if capture_stdout:
-            self.options = dict(options, capture_stdout=True)
-        else:
-            self.options = options
+        self.content = content
+        self.options = dict(options, capture_stdout=True) if capture_stdout else options
         self._ui = ui
         self._config = config
         self._is_windows = sys.platform == "win32"
         self.invocation = invocation
-        self.named_args = self._parse_named_args(invocation[1:])
+        self.inheritance = inheritance or TaskInheritance(cwd=str(config.project_dir))
 
     @classmethod
     def from_config(
@@ -101,6 +114,7 @@ class PoeTask(metaclass=MetaPoeTask):
         ui: "PoeUi",
         invocation: Tuple[str, ...],
         capture_stdout: Optional[bool] = None,
+        inheritance: Optional[TaskInheritance] = None,
     ) -> "PoeTask":
         task_def = config.tasks.get(task_name)
         if not task_def:
@@ -112,6 +126,7 @@ class PoeTask(metaclass=MetaPoeTask):
             ui,
             invocation=invocation,
             capture_stdout=capture_stdout,
+            inheritance=inheritance,
         )
 
     @classmethod
@@ -124,6 +139,7 @@ class PoeTask(metaclass=MetaPoeTask):
         invocation: Tuple[str, ...],
         array_item: Union[bool, str] = False,
         capture_stdout: Optional[bool] = None,
+        inheritance: Optional[TaskInheritance] = None,
     ) -> "PoeTask":
         task_type = cls.resolve_task_type(task_def, config, array_item)
         if task_type is None:
@@ -137,13 +153,8 @@ class PoeTask(metaclass=MetaPoeTask):
             options["capture_stdout"] = capture_stdout
 
         if isinstance(task_def, (str, list)):
-            return cls.__task_types[task_type](
-                name=task_name,
-                content=task_def,
-                options=options,
-                ui=ui,
-                config=config,
-                invocation=invocation,
+            task_def = cls.normalize_task_def(
+                task_def, config, task_type=cls.__task_types[task_type]
             )
 
         assert isinstance(task_def, dict)
@@ -156,8 +167,28 @@ class PoeTask(metaclass=MetaPoeTask):
             ui=ui,
             config=config,
             invocation=invocation,
+            inheritance=inheritance,
         )
 
+    @classmethod
+    def normalize_task_def(
+        cls,
+        task_def: TaskDef,
+        config: "PoeConfig",
+        *,
+        task_type: Optional[Type["PoeTask"]] = None,
+        array_item: Union[bool, str] = False,
+    ):
+        if isinstance(task_def, dict):
+            return task_def
+
+        if not task_type:
+            task_type_key = cls.resolve_task_type(task_def, config, array_item)
+            assert task_type_key
+            task_type = cls.__task_types[task_type_key]
+
+        return {getattr(task_type, "__key__", "__key_unknown__"): task_def}
+
     @classmethod
     def resolve_task_type(
         cls,
@@ -175,74 +206,118 @@ class PoeTask(metaclass=MetaPoeTask):
             else:
                 return config.default_task_type
 
-        elif isinstance(task_def, list):
-            return config.default_array_task_type
-
         elif isinstance(task_def, dict):
             task_type_keys = set(task_def.keys()).intersection(cls.__task_types)
             if len(task_type_keys) == 1:
                 return next(iter(task_type_keys))
 
+        elif isinstance(task_def, list):
+            return config.default_array_task_type
+
         return None
 
-    def _parse_named_args(self, extra_args: Sequence[str]) -> Optional[Dict[str, str]]:
-        args_def = self.options.get("args")
-        if args_def:
-            return PoeTaskArgs(args_def, self.name).parse(extra_args)
-        return None
+    def get_parsed_arguments(
+        self, env: "EnvVarsManager"
+    ) -> Tuple[Dict[str, str], Tuple[str, ...]]:
+        if self._parsed_args is None:
+            all_args = self.invocation[1:]
 
-    def add_named_args_to_env(
-        self, env: Mapping[str, str]
-    ) -> Tuple[Mapping[str, str], bool]:
-        if self.named_args is None:
-            return env, False
-        return (
-            dict(
-                env,
-                **(
-                    {
-                        key: str(value)
-                        for key, value in self.named_args.items()
-                        if value is not None
-                    }
-                ),
-            ),
-            bool(self.named_args),
-        )
+            if args_def := self.options.get("args"):
+                from .args import PoeTaskArgs
+
+                try:
+                    split_index = all_args.index("--")
+                    option_args = all_args[:split_index]
+                    extra_args = all_args[split_index + 1 :]
+                except ValueError:
+                    option_args = all_args
+                    extra_args = tuple()
+
+                self._parsed_args = (
+                    PoeTaskArgs(args_def, self.name, self._ui.program_name, env).parse(
+                        option_args
+                    ),
+                    extra_args,
+                )
+
+            else:
+                self._parsed_args = ({}, all_args)
+
+        return self._parsed_args
 
     def run(
         self,
         context: "RunContext",
-        extra_args: Sequence[str] = tuple(),
-        env: Optional[Mapping[str, str]] = None,
+        parent_env: Optional["EnvVarsManager"] = None,
     ) -> int:
         """
         Run this task
         """
+
+        if environ.get("POE_DEBUG"):
+            task_type_key = self.__key__  # type: ignore[attr-defined]
+            print(f" * Running     {task_type_key}:{self.name}")
+            print(f" . Invocation  {self.invocation!r}")
+
         upstream_invocations = self._get_upstream_invocations(context)
-        return self._handle_run(
-            context,
-            extra_args,
-            context.get_env(
-                env,
-                self.options.get("envfile"),
-                self.options.get("env"),
-                upstream_invocations["uses"],
-            ),
+
+        if context.dry and upstream_invocations.get("uses", {}):
+            self._print_action(
+                (
+                    "unresolved dependency task results via uses option for task "
+                    f"{self.name!r}"
+                ),
+                dry=True,
+                unresolved=True,
+            )
+            return 0
+
+        task_env = context.get_task_env(
+            parent_env,
+            self.options.get("envfile"),
+            self.options.get("env"),
+            upstream_invocations["uses"],
         )
 
+        if environ.get("POE_DEBUG"):
+            named_arg_values, extra_args = self.get_parsed_arguments(task_env)
+            print(f" . Parsed args {named_arg_values!r}")
+            print(f" . Extra args  {extra_args!r}")
+
+        return self._handle_run(context, task_env)
+
     def _handle_run(
         self,
         context: "RunContext",
-        extra_args: Sequence[str],
-        env: Mapping[str, str],
+        env: "EnvVarsManager",
     ) -> int:
         """
-        _handle_run must be implemented by a subclass and return a single executor
+        This method must be implemented by a subclass and return a single executor
         result.
         """
         raise NotImplementedError
 
+    def _get_executor(self, context: "RunContext", env: "EnvVarsManager"):
+        return context.get_executor(
+            self.invocation,
+            env,
+            working_dir=self.get_working_dir(env),
+            executor_config=self.options.get("executor"),
+            capture_stdout=self.options.get("capture_stdout", False),
+        )
+
+    def get_working_dir(
+        self,
+        env: "EnvVarsManager",
+    ) -> Path:
+        cwd_option = env.fill_template(self.options.get("cwd", self.inheritance.cwd))
+        working_dir = Path(cwd_option)
+
+        if not working_dir.is_absolute():
+            working_dir = self._config.project_dir / working_dir
+
+        return working_dir
+
     def iter_upstream_tasks(
         self, context: "RunContext"
     ) -> Iterator[Tuple[str, "PoeTask"]]:
@@ -256,23 +331,24 @@ class PoeTask(metaclass=MetaPoeTask):
         """
         NB. this memoization assumes the context (and contained env vars) will be the
         same in all instances for the lifetime of this object. Whilst this should be OK
-        for all corrent usecases is it strictly speaking something that this object
+        for all current usecases is it strictly speaking something that this object
         should not know enough to safely assume. So we probably want to revisit this.
         """
+        import shlex
+
         if self.__upstream_invocations is None:
-            env: Mapping
-            env = context.get_env(
-                {}, self.options.get("envfile"), self.options.get("env")
+            env = context.get_task_env(
+                None, self.options.get("envfile"), self.options.get("env")
             )
-            env, _ = self.add_named_args_to_env(env)
+            env.update(self.get_parsed_arguments(env)[0])
 
             self.__upstream_invocations = {
                 "deps": [
-                    tuple(shlex.split(resolve_envvars(task_ref, env)))
+                    tuple(shlex.split(env.fill_template(task_ref)))
                     for task_ref in self.options.get("deps", tuple())
                 ],
                 "uses": {
-                    key: tuple(shlex.split(resolve_envvars(task_ref, env)))
+                    key: tuple(shlex.split(env.fill_template(task_ref)))
                     for key, task_ref in self.options.get("uses", {}).items()
                 },
             }
@@ -295,23 +371,32 @@ class PoeTask(metaclass=MetaPoeTask):
 
     @classmethod
     def validate_def(
-        cls, task_name: str, task_def: TaskDef, config: "PoeConfig"
+        cls,
+        task_name: str,
+        task_def: TaskDef,
+        config: "PoeConfig",
+        *,
+        anonymous: bool = False,
+        extra_options: Collection[str] = tuple(),
     ) -> Optional[str]:
         """
         Check the given task name and definition for validity and return a message
         describing the first encountered issue if any.
+        If anonymous is set to True then task_name is not validated.
         """
-        if not (task_name[0].isalpha() or task_name[0] == "_"):
+        if not anonymous and (not (task_name[0].isalpha() or task_name[0] == "_")):
             return (
                 f"Invalid task name: {task_name!r}. Task names must start with a letter"
                 " or underscore."
             )
-        elif not _TASK_NAME_PATTERN.match(task_name):
+
+        if not anonymous and not _TASK_NAME_PATTERN.match(task_name):
             return (
                 f"Invalid task name: {task_name!r}. Task names characters must be "
                 "alphanumeric, colon, underscore or dash."
             )
-        elif isinstance(task_def, dict):
+
+        if isinstance(task_def, dict):
             task_type_keys = set(task_def.keys()).intersection(cls.__task_types)
             if len(task_type_keys) != 1:
                 return (
@@ -323,42 +408,59 @@ class PoeTask(metaclass=MetaPoeTask):
             task_type = cls.__task_types[task_type_key]
             if not isinstance(task_content, task_type.__content_type__):
                 return (
-                    f"Invalid task: {task_name!r}. {task_type} value must be a "
-                    f"{task_type.__content_type__}"
+                    f"Invalid task: {task_name!r}. Content for {task_type.__name__} "
+                    f"must be a {task_type.__content_type__.__name__}"
                 )
-            else:
-                for key in set(task_def) - {task_type_key}:
-                    expected_type = cls.__base_options.get(
-                        key, task_type.__options__.get(key)
-                    )
-                    if expected_type is None:
+
+            for key in set(task_def) - {task_type_key}:
+                expected_type = cls.__base_options.get(
+                    key, task_type.__options__.get(key)
+                )
+                if expected_type is None:
+                    if key not in extra_options:
                         return (
                             f"Invalid task: {task_name!r}. Unrecognised option "
                             f"{key!r} for task of type: {task_type_key}."
                         )
-                    elif not isinstance(task_def[key], expected_type):
-                        return (
-                            f"Invalid task: {task_name!r}. Option {key!r} should "
-                            f"have a value of type {expected_type!r}"
-                        )
-                else:
-                    if hasattr(task_type, "_validate_task_def"):
-                        task_type_issue = task_type._validate_task_def(
-                            task_name, task_def, config
-                        )
-                        if task_type_issue:
-                            return task_type_issue
+                elif not isinstance(task_def[key], expected_type):
+                    return (
+                        f"Invalid task: {task_name!r}. Option {key!r} should "
+                        f"have a value of type {expected_type!r}"
+                    )
+            else:
+                if hasattr(task_type, "_validate_task_def"):
+                    task_type_issue = task_type._validate_task_def(
+                        task_name, task_def, config
+                    )
+                    if task_type_issue:
+                        return task_type_issue
 
             if "args" in task_def:
+                from .args import PoeTaskArgs
+
                 return PoeTaskArgs.validate_def(task_name, task_def["args"])
 
+            if "cwd" in task_def:
+                path = Path(config.project_dir).joinpath(task_def["cwd"]).resolve()
+                if not str(path).startswith(config.project_dir):
+                    return (
+                        f"Invalid task: {task_name!r}. cwd option may not specify a "
+                        "directory outside of the project."
+                    )
+
             if "\n" in task_def.get("help", ""):
                 return (
                     f"Invalid task: {task_name!r}. Help messages cannot contain "
                     "line breaks"
                 )
 
-            all_task_names = set(config.tasks)
+            if task_def.get("use_exec") and task_def.get("capture_stdout"):
+                return (
+                    f"Invalid task: {task_name!r}, 'use_exec' and 'capture_stdout'"
+                    " options cannot be both provided on the same task."
+                )
+
+            all_task_names = set(config.tasks.keys())
 
             if "deps" in task_def:
                 for dep in task_def["deps"]:
@@ -369,13 +471,26 @@ class PoeTask(metaclass=MetaPoeTask):
                             f"reference to unknown task: {dep_task_name!r}"
                         )
 
+                    referenced_task = config.tasks[dep_task_name]
+                    if isinstance(referenced_task, dict) and referenced_task.get(
+                        "use_exec"
+                    ):
+                        return (
+                            f"Invalid task: {task_name!r}. deps options contains "
+                            "reference to task with use_exec set to true: "
+                            f"{dep_task_name!r}"
+                        )
+
             if "uses" in task_def:
+                from ..helpers import is_valid_env_var
+
                 for key, dep in task_def["uses"].items():
                     if not is_valid_env_var(key):
                         return (
                             f"Invalid task: {task_name!r} uses options contains invalid"
                             f" key: {key!r}"
                         )
+
                     dep_task_name = dep.split(" ", 1)[0]
                     if dep_task_name not in all_task_names:
                         return (
@@ -383,6 +498,34 @@ class PoeTask(metaclass=MetaPoeTask):
                             f"reference to unknown task: {dep_task_name!r}"
                         )
 
+                    referenced_task = config.tasks[dep_task_name]
+                    if isinstance(referenced_task, dict):
+                        if referenced_task.get("use_exec"):
+                            return (
+                                f"Invalid task: {task_name!r}, 'uses' option references"
+                                " task with 'use_exec' set to true: "
+                                f"{dep_task_name!r}"
+                            )
+                        if referenced_task.get("capture_stdout"):
+                            return (
+                                f"Invalid task: {task_name!r}, 'uses' option references"
+                                " task with 'capture_stdout' option set: "
+                                f"{dep_task_name!r}"
+                            )
+
+        elif isinstance(task_def, list):
+            task_type_key = config.default_array_task_type
+            task_type = cls.__task_types[task_type_key]
+            normalized_task_def = cls.normalize_task_def(
+                task_def, config, task_type=task_type
+            )
+            if hasattr(task_type, "_validate_task_def"):
+                task_type_issue = task_type._validate_task_def(
+                    task_name, normalized_task_def, config
+                )
+                if task_type_issue:
+                    return task_type_issue
+
         return None
 
     @classmethod
@@ -421,12 +564,14 @@ class PoeTask(metaclass=MetaPoeTask):
         issue = None
         return issue
 
-    def _print_action(self, action: str, dry: bool):
+    def _print_action(self, action: str, dry: bool, unresolved: bool = False):
         """
         Print the action taken by a task just before executing it.
         """
         min_verbosity = -1 if dry else 0
-        arrow = "<=" if self.options.get("capture_stdout") else "=>"
+        arrow = (
+            "??" if unresolved else "<=" if self.options.get("capture_stdout") else "=>"
+        )
         self._ui.print_msg(
             f"<hl>Poe {arrow}</hl> <action>{action}</action>", min_verbosity
         )
