@@ -1,22 +1,21 @@
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     List,
-    Mapping,
     Optional,
-    Sequence,
     Tuple,
     Type,
-    TYPE_CHECKING,
     Union,
 )
-from .base import PoeTask, TaskContent
+
 from ..exceptions import ExecutionError, PoeException
+from .base import PoeTask, TaskContent, TaskInheritance
 
 if TYPE_CHECKING:
     from ..config import PoeConfig
     from ..context import RunContext
-    from ..executor import PoeExecutor
+    from ..env.manager import EnvVarsManager
     from ..ui import PoeUi
 
 
@@ -43,9 +42,12 @@ class SequenceTask(PoeTask):
         config: "PoeConfig",
         invocation: Tuple[str, ...],
         capture_stdout: bool = False,
+        inheritance: Optional[TaskInheritance] = None,
     ):
-        assert capture_stdout == False
-        super().__init__(name, content, options, ui, config, invocation)
+        assert capture_stdout is False
+        super().__init__(
+            name, content, options, ui, config, invocation, False, inheritance
+        )
 
         self.subtasks = [
             self.from_def(
@@ -55,20 +57,23 @@ class SequenceTask(PoeTask):
                 invocation=(task_name,),
                 ui=ui,
                 array_item=self.options.get("default_item_type", True),
+                inheritance=TaskInheritance.from_task(self),
             )
             for index, item in enumerate(self.content)
-            for task_name in (item if isinstance(item, str) else f"{name}[{index}]",)
+            for task_name in (
+                item if isinstance(item, str) else self._subtask_name(name, index),
+            )
         ]
 
     def _handle_run(
         self,
         context: "RunContext",
-        extra_args: Sequence[str],
-        env: Mapping[str, str],
+        env: "EnvVarsManager",
     ) -> int:
-        env, has_named_args = self.add_named_args_to_env(env)
+        named_arg_values, extra_args = self.get_parsed_arguments(env)
+        env.update(named_arg_values)
 
-        if not has_named_args and any(arg.strip() for arg in extra_args):
+        if not named_arg_values and any(arg.strip() for arg in self.invocation[1:]):
             raise PoeException(f"Sequence task {self.name!r} does not accept arguments")
 
         if len(self.subtasks) > 1:
@@ -78,7 +83,7 @@ class SequenceTask(PoeTask):
         ignore_fail = self.options.get("ignore_fail")
         non_zero_subtasks: List[str] = list()
         for subtask in self.subtasks:
-            task_result = subtask.run(context=context, extra_args=tuple(), env=env)
+            task_result = subtask.run(context=context, parent_env=env)
             if task_result and not ignore_fail:
                 raise ExecutionError(
                     f"Sequence aborted after failed subtask {subtask.name!r}"
@@ -92,6 +97,10 @@ class SequenceTask(PoeTask):
             )
         return 0
 
+    @classmethod
+    def _subtask_name(cls, task_name: str, index: int):
+        return f"{task_name}[{index}]"
+
     @classmethod
     def _validate_task_def(
         cls, task_name: str, task_def: Dict[str, Any], config: "PoeConfig"
@@ -104,6 +113,7 @@ class SequenceTask(PoeTask):
                 "Unsupported value for option `default_item_type` for task "
                 f"{task_name!r}. Expected one of {cls.get_task_types(content_type=str)}"
             )
+
         ignore_fail = task_def.get("ignore_fail")
         if ignore_fail is not None and ignore_fail not in (
             True,
@@ -112,8 +122,39 @@ class SequenceTask(PoeTask):
             "return_non_zero",
         ):
             return (
-                "Unsupported value for option `ignore_fail` for task "
-                f'{task_name!r}. Expected one of (true, false, "return_zero", "return_non_zero")'
+                f"Unsupported value for option `ignore_fail` for task {task_name!r}."
+                ' Expected one of (true, false, "return_zero", "return_non_zero")'
             )
 
+        for index, task_item in enumerate(task_def["sequence"]):
+            if isinstance(task_item, dict):
+                if len(task_item.get("args", tuple())):
+                    return (
+                        "Unsupported option `args` for task declared inside sequence "
+                        f"task {task_name!r}."
+                    )
+
+                subtask_issue = cls.validate_def(
+                    cls._subtask_name(task_name, index),
+                    task_item,
+                    config,
+                    anonymous=True,
+                )
+                if subtask_issue:
+                    return subtask_issue
+
+            else:
+                subtask_issue = cls.validate_def(
+                    cls._subtask_name(task_name, index),
+                    cls.normalize_task_def(
+                        task_item,
+                        config,
+                        array_item=default_item_type or True,
+                    ),
+                    config,
+                    anonymous=True,
+                )
+                if subtask_issue:
+                    return subtask_issue
+
         return None
