@@ -1,24 +1,23 @@
+import re
 from os import environ
-from shutil import which
-import sys
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     List,
-    Mapping,
     Optional,
-    Sequence,
     Tuple,
     Type,
-    TYPE_CHECKING,
     Union,
 )
+
 from ..exceptions import PoeException
 from .base import PoeTask
 
 if TYPE_CHECKING:
     from ..config import PoeConfig
     from ..context import RunContext
+    from ..env.manager import EnvVarsManager
 
 
 class ShellTask(PoeTask):
@@ -34,12 +33,12 @@ class ShellTask(PoeTask):
     def _handle_run(
         self,
         context: "RunContext",
-        extra_args: Sequence[str],
-        env: Mapping[str, str],
+        env: "EnvVarsManager",
     ) -> int:
-        env, has_named_args = self.add_named_args_to_env(env)
+        named_arg_values, extra_args = self.get_parsed_arguments(env)
+        env.update(named_arg_values)
 
-        if not has_named_args and any(arg.strip() for arg in extra_args):
+        if not named_arg_values and any(arg.strip() for arg in self.invocation[1:]):
             raise PoeException(f"Shell task {self.name!r} does not accept arguments")
 
         interpreter_cmd = self.resolve_interpreter_cmd()
@@ -55,9 +54,12 @@ class ShellTask(PoeTask):
                 message += "Some dependencies may be missing from your system."
             raise PoeException(message)
 
-        self._print_action(self.content, context.dry)
-        return context.get_executor(self.invocation, env, self.options).execute(
-            interpreter_cmd, input=self.content.encode()
+        content = _unindent_code(self.content).rstrip()
+
+        self._print_action(content, context.dry)
+
+        return self._get_executor(context, env).execute(
+            interpreter_cmd, input=content.encode()
         )
 
     def _get_interpreter_config(self) -> Tuple[str, ...]:
@@ -76,13 +78,18 @@ class ShellTask(PoeTask):
         for item in self._get_interpreter_config():
             executable = self._locate_interpreter(item)
             if executable is None:
-                return None
+                continue
+
             if item in ("pwsh", "powershell"):
                 return [executable, "-NoLogo", "-Command", "-"]
+
             return [executable]
+
         return None
 
     def _locate_interpreter(self, interpreter: str) -> Optional[str]:
+        from shutil import which
+
         result = None
         prog_files = environ.get("PROGRAMFILES", "C:\\Program Files")
 
@@ -102,12 +109,21 @@ class ShellTask(PoeTask):
         elif interpreter == "sh":
             result = which("sh") or which("/bin/sh")
 
-        elif interpreter == "bash":
-            result = which("bash") or which("/bin/bash")
-
-            # Specifically look for git bash on windows
+            # Specifically look for git sh on windows
             if result is None and self._is_windows:
-                result = which(f"{prog_files}\\Git\\bin\\bash.exe")
+                result = which(f"{prog_files}\\Git\\bin\\sh.exe")
+
+        elif interpreter == "bash":
+            if self._is_windows:
+                # Specifically look for git bash on windows as the preferred option
+                # Don't trust bash from the path becuase it might be a useless decoy
+                result = (
+                    which(f"{prog_files}\\Git\\bin\\bash.exe")
+                    or which("/bin/bash")
+                    or which("bash")
+                )
+            else:
+                result = which("bash") or which("/bin/bash")
 
         elif interpreter == "zsh":
             result = which("zsh") or which("/bin/zsh")
@@ -131,7 +147,8 @@ class ShellTask(PoeTask):
                 )
 
         elif interpreter == "python":
-            result = which("python") or which("python3") or sys.executable
+            # Exactly which python executable to use is usually resolved by the executor
+            result = "python"
 
         return result
 
@@ -162,3 +179,36 @@ class ShellTask(PoeTask):
                     )
 
         return None
+
+
+def _unindent_code(python_code: str):
+    """
+    Unindent all lines by the indent level of the first line.
+    This is rather naive, but should usually work as one would naively expect for a
+    multiline script in a multiline string value in toml.
+
+    It will not always work correctly if the multiline string itself contains a triple
+    quoted multiline python string or similar. Let's say that's OK for now.
+    """
+
+    if not python_code.startswith(" "):
+        return python_code
+
+    indent = 0
+    while python_code[indent] == " ":
+        indent += 1
+
+    prefix = " " * indent
+    return "\n".join(
+        _remove_prefix(line, prefix)
+        for line in re.split(r"(?:\r\n|\r|\n)", python_code)
+    )
+
+
+def _remove_prefix(text: str, prefix: str):
+    """
+    When we drop support for python <3.9 then str.removeprefix can be used
+    """
+    if text.startswith(prefix):
+        return text[len(prefix) :]
+    return text
