@@ -1,13 +1,21 @@
-import argparse
 import os
-from pastel import Pastel
 import sys
-from typing import IO, List, Mapping, Optional, Sequence, Tuple, Union
-from .exceptions import PoeException
+from typing import IO, TYPE_CHECKING, List, Mapping, Optional, Sequence, Tuple, Union
+
 from .__version__ import __version__
+from .exceptions import ExecutionError, PoeException
+
+if TYPE_CHECKING:
+    from argparse import ArgumentParser, Namespace
+
+    from pastel import Pastel
 
 
 def guess_ansi_support(file):
+    if os.environ.get("NO_COLOR", "0")[0] != "0":
+        # https://no-color.org/
+        return False
+
     return (
         (sys.platform != "win32" or "ANSICON" in os.environ)
         and hasattr(file, "isatty")
@@ -19,14 +27,17 @@ STDOUT_ANSI_SUPPORT = guess_ansi_support(sys.stdout)
 
 
 class PoeUi:
-    args: argparse.Namespace
-    _color: Pastel
+    args: "Namespace"
+    _color: "Pastel"
 
-    def __init__(self, output: IO):  # TODO: make IO wrapper
+    def __init__(self, output: IO, program_name: str = "poe"):
         self.output = output
+        self.program_name = program_name
         self._init_colors()
 
     def _init_colors(self):
+        from pastel import Pastel
+
         self._color = Pastel(guess_ansi_support(self.output))
         self._color.add_style("u", "default", options="underline")
         self._color.add_style("hl", "light_gray")
@@ -42,9 +53,11 @@ class PoeUi:
         """Provide easy access to arguments"""
         return getattr(self.args, key, None)
 
-    def build_parser(self) -> argparse.ArgumentParser:
+    def build_parser(self) -> "ArgumentParser":
+        import argparse
+
         parser = argparse.ArgumentParser(
-            prog="poe",
+            prog=self.program_name,
             description="Poe the Poet: A task runner that works well with poetry.",
             add_help=False,
             allow_abbrev=False,
@@ -91,7 +104,7 @@ class PoeUi:
             dest="dry_run",
             action="store_true",
             default=False,
-            help="Print the task contents but don't actaully run it",
+            help="Print the task contents but don't actually run it",
         )
 
         parser.add_argument(
@@ -134,7 +147,9 @@ class PoeUi:
 
     def print_help(
         self,
-        tasks: Optional[Mapping[str, Tuple[str, Sequence[Tuple[str, str]]]]] = None,
+        tasks: Optional[
+            Mapping[str, Tuple[str, Sequence[Tuple[Tuple[str, ...], str, str]]]]
+        ] = None,
         info: Optional[str] = None,
         error: Optional[PoeException] = None,
     ):
@@ -157,16 +172,18 @@ class PoeUi:
 
         if error:
             # TODO: send this to stderr instead?
-            result.append([f"<error>Error: {error.msg} </error>"])
+            error_line = [f"<error>Error: {error.msg} </error>"]
             if error.cause:
-                result[-1].append(f"<error> From: {error.cause} </error>")  # type: ignore
+                error_line.append(f"<error> From: {error.cause} </error>")
+            result.append(error_line)
 
         if verbosity >= 0:
-            # Use argparse for usage summary
             result.append(
                 (
                     "<h2>USAGE</h2>",
-                    "  <u>poe</u> [-h] [-v | -q] [--root PATH] [--ansi | --no-ansi] task [task arguments]",
+                    f"  <u>{self.program_name}</u>"
+                    " [-h] [-v | -q] [--root PATH] [--ansi | --no-ansi]"
+                    " task [task arguments]",
                 )
             )
 
@@ -184,11 +201,13 @@ class PoeUi:
                 max_task_len = max(
                     max(
                         len(task),
-                        max([len(", ".join(opts)) for (opts, _) in args] or (0,)) + 2,
+                        max([len(", ".join(opts)) for (opts, _, _) in args] or (0,))
+                        + 2,
                     )
                     for task, (_, args) in tasks.items()
                 )
                 col_width = max(13, min(30, max_task_len))
+
                 tasks_section = ["<h2>CONFIGURED TASKS</h2>"]
                 for task, (help_text, args_help) in tasks.items():
                     if task.startswith("_"):
@@ -196,13 +215,19 @@ class PoeUi:
                     tasks_section.append(
                         f"  <em>{self._padr(task, col_width)}</em>  {help_text}"
                     )
-                    for (options, arg_help_text) in args_help:
-                        tasks_section.append(
-                            "    "
-                            f"<em3>{self._padr(', '.join(options), col_width - 2)}</em3>"
-                            f"  {arg_help_text}"
-                        )
+                    for options, arg_help_text, default in args_help:
+                        task_arg_help = [
+                            "   ",
+                            f"<em3>{self._padr(', '.join(options), col_width-1)}</em3>",
+                        ]
+                        if arg_help_text:
+                            task_arg_help.append(arg_help_text)
+                        if default:
+                            task_arg_help.append(default)
+                        tasks_section.append(" ".join(task_arg_help))
+
                 result.append(tasks_section)
+
             else:
                 result.append("<h2-dim>NO TASKS CONFIGURED</h2-dim>")
 
@@ -211,7 +236,7 @@ class PoeUi:
                 section if isinstance(section, str) else "\n".join(section).strip("\n")
                 for section in result
             )
-            + f"\n"
+            + "\n"
             + ("\n" if verbosity >= 0 else "")
         )
 
@@ -225,14 +250,14 @@ class PoeUi:
         if verbosity <= self.verbosity:
             self._print(message, end=end)
 
-    def print_error(self, error: Exception):
-        self._print(f"<error>Error: {error.msg} </error>")  # type: ignore
-        if error.cause:  # type: ignore
-            self._print(f"<error> From: {error.cause} </error>")  # type: ignore
+    def print_error(self, error: Union[PoeException, ExecutionError]):
+        self._print(f"<error>Error: {error.msg} </error>")
+        if error.cause:
+            self._print(f"<error> From: {error.cause} </error>")
 
     def print_version(self):
         if self.verbosity >= 0:
-            result = f"Poe the poet - version: <em>{__version__}</em>\n"
+            result = f"Poe the Poet - version: <em>{__version__}</em>\n"
         else:
             result = f"{__version__}\n"
         self._print(result)
